import { ArgumentInvalidException, ValueObject } from '@mobicoop/ddd-library';
import { Role } from '../ad.types';
import { Target } from '../candidate.types';
import { Actor, ActorProps } from './actor.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface ActorTimeProps extends ActorProps {
  firstDatetime: Date;
  firstMinDatetime: Date;
  firstMaxDatetime: Date;
  lastDatetime: Date;
  lastMinDatetime: Date;
  lastMaxDatetime: Date;
}

export class ActorTime extends ValueObject<ActorTimeProps> {
  get role(): Role {
    return this.props.role;
  }

  get target(): Target {
    return this.props.target;
  }

  get firstDatetime(): Date {
    return this.props.firstDatetime;
  }

  get firstMinDatetime(): Date {
    return this.props.firstMinDatetime;
  }

  get firstMaxDatetime(): Date {
    return this.props.firstMaxDatetime;
  }

  get lastDatetime(): Date {
    return this.props.lastDatetime;
  }

  get lastMinDatetime(): Date {
    return this.props.lastMinDatetime;
  }

  get lastMaxDatetime(): Date {
    return this.props.lastMaxDatetime;
  }

  protected validate(props: ActorTimeProps): void {
    // validate actor props
    new Actor({
      role: props.role,
      target: props.target,
    });
    if (props.firstDatetime.getUTCDay() != props.lastDatetime.getUTCDay())
      throw new ArgumentInvalidException(
        'firstDatetime week day must be equal to lastDatetime week day',
      );
    if (props.firstDatetime > props.lastDatetime)
      throw new ArgumentInvalidException(
        'firstDatetime must be before or equal to lastDatetime',
      );
    if (props.firstMinDatetime > props.firstDatetime)
      throw new ArgumentInvalidException(
        'firstMinDatetime must be before or equal to firstDatetime',
      );
    if (props.firstDatetime > props.firstMaxDatetime)
      throw new ArgumentInvalidException(
        'firstDatetime must be before or equal to firstMaxDatetime',
      );
    if (props.lastMinDatetime > props.lastDatetime)
      throw new ArgumentInvalidException(
        'lastMinDatetime must be before or equal to lastDatetime',
      );
    if (props.lastDatetime > props.lastMaxDatetime)
      throw new ArgumentInvalidException(
        'lastDatetime must be before or equal to lastMaxDatetime',
      );
  }
}
