import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { Actor } from './actor.value-object';
import { Role } from '../ad.types';
import { Point, PointProps } from './point.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface CarpoolPathItemProps extends PointProps {
  actors: Actor[];
}

export class CarpoolPathItem extends ValueObject<CarpoolPathItemProps> {
  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  get actors(): Actor[] {
    return this.props.actors;
  }

  protected validate(props: CarpoolPathItemProps): void {
    // validate point props
    new Point({
      lon: props.lon,
      lat: props.lat,
    });
    if (props.actors.length <= 0)
      throw new ArgumentOutOfRangeException('at least one actor is required');
    if (
      props.actors.filter((actor: Actor) => actor.role == Role.DRIVER).length >
      1
    )
      throw new ArgumentOutOfRangeException(
        'a carpoolStep can contain only one driver',
      );
  }
}
