import { JourneyCompleter } from '@modules/ad/core/application/queries/match/completer/journey.completer';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { Actor } from '@modules/ad/core/domain/value-objects/actor.value-object';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery(
  {
    algorithmType: AlgorithmType.PASSENGER_ORIENTED,
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-08-28',
    toDate: '2023-08-28',
    schedule: [
      {
        time: '07:05',
      },
    ],
    strict: false,
    waypoints: [originWaypoint, destinationWaypoint],
  },
  {
    getBasic: jest.fn().mockImplementation(() => ({
      distance: 350101,
      duration: 14422,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336544,
      points: [],
    })),
    getDetailed: jest.fn().mockImplementation(() => ({
      distance: 350102,
      duration: 14423,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336544,
      points: [],
    })),
  },
);

const candidate: CandidateEntity = CandidateEntity.create({
  id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
  role: Role.DRIVER,
  dateInterval: {
    lowerDate: '2023-08-28',
    higherDate: '2023-08-28',
  },
  driverWaypoints: [
    {
      lat: 48.678454,
      lon: 6.189745,
    },
    {
      lat: 48.84877,
      lon: 2.398457,
    },
  ],
  passengerWaypoints: [
    {
      lat: 48.689445,
      lon: 6.17651,
    },
    {
      lat: 48.8566,
      lon: 2.3522,
    },
  ],
  driverDistance: 350145,
  driverDuration: 13548,
  driverSchedule: [
    {
      day: 1,
      time: '07:00',
      margin: 900,
    },
  ],
  passengerSchedule: [
    {
      day: 1,
      time: '07:10',
      margin: 900,
    },
  ],
  spacetimeDetourRatio: {
    maxDistanceDetourRatio: 0.3,
    maxDurationDetourRatio: 0.3,
  },
}).setCarpoolPath([
  {
    lat: 48.689445,
    lon: 6.17651,
    actors: [
      new Actor({
        role: Role.DRIVER,
        target: Target.START,
      }),
      new Actor({
        role: Role.PASSENGER,
        target: Target.START,
      }),
    ],
  },
  {
    lat: 48.8566,
    lon: 2.3522,
    actors: [
      new Actor({
        role: Role.DRIVER,
        target: Target.FINISH,
      }),
      new Actor({
        role: Role.PASSENGER,
        target: Target.FINISH,
      }),
    ],
  },
]);
candidate.createJourneys = jest.fn().mockImplementation(() => candidate);

describe('Journey completer', () => {
  it('should complete candidates with their journey', async () => {
    const journeyCompleter: JourneyCompleter = new JourneyCompleter(matchQuery);
    const completedCandidates: CandidateEntity[] =
      await journeyCompleter.complete([candidate]);
    expect(completedCandidates.length).toBe(1);
  });
});
