import { PassengerOrientedGeoFilter } from '@modules/ad/core/application/queries/match/filter/passenger-oriented-geo.filter';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery(
  {
    algorithmType: AlgorithmType.PASSENGER_ORIENTED,
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-08-28',
    toDate: '2023-08-28',
    schedule: [
      {
        time: '07:05',
      },
    ],
    strict: false,
    waypoints: [originWaypoint, destinationWaypoint],
  },
  {
    getBasic: jest.fn(),
    getDetailed: jest.fn(),
  },
);

const candidate: CandidateEntity = CandidateEntity.create({
  id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
  role: Role.DRIVER,
  dateInterval: {
    lowerDate: '2023-08-28',
    higherDate: '2023-08-28',
  },
  driverWaypoints: [
    {
      lat: 48.678454,
      lon: 6.189745,
    },
    {
      lat: 48.84877,
      lon: 2.398457,
    },
  ],
  passengerWaypoints: [
    {
      lat: 48.689445,
      lon: 6.17651,
    },
    {
      lat: 48.8566,
      lon: 2.3522,
    },
  ],
  driverDistance: 350145,
  driverDuration: 13548,
  driverSchedule: [
    {
      day: 0,
      time: '07:00',
      margin: 900,
    },
  ],
  passengerSchedule: [
    {
      day: 0,
      time: '07:10',
      margin: 900,
    },
  ],
  spacetimeDetourRatio: {
    maxDistanceDetourRatio: 0.3,
    maxDurationDetourRatio: 0.3,
  },
});

describe('Passenger oriented geo filter', () => {
  it('should not filter valid candidates', async () => {
    const passengerOrientedGeoFilter: PassengerOrientedGeoFilter =
      new PassengerOrientedGeoFilter(matchQuery);
    candidate.isDetourValid = () => true;
    const filteredCandidates: CandidateEntity[] =
      await passengerOrientedGeoFilter.filter([candidate]);
    expect(filteredCandidates.length).toBe(1);
  });
  it('should filter invalid candidates', async () => {
    const passengerOrientedGeoFilter: PassengerOrientedGeoFilter =
      new PassengerOrientedGeoFilter(matchQuery);
    candidate.isDetourValid = () => false;
    const filteredCandidates: CandidateEntity[] =
      await passengerOrientedGeoFilter.filter([candidate]);
    expect(filteredCandidates.length).toBe(0);
  });
});
