import { IsAfterOrEqual } from '@modules/ad/interface/grpc-controllers/dtos/validators/decorators/is-after-or-equal.decorator';
import { Validator } from 'class-validator';

describe('Is after or equal decorator', () => {
  class MyClass {
    firstDate: string;

    @IsAfterOrEqual('firstDate', {
      message: 'secondDate must be after or equal to firstDate',
    })
    secondDate: string;
  }

  it('should return a property decorator has a function', () => {
    const isAfterOrEqual = IsAfterOrEqual('someProperty');
    expect(typeof isAfterOrEqual).toBe('function');
  });

  it('should validate a secondDate posterior to firstDate', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.firstDate = '2023-07-20';
    myClassInstance.secondDate = '2023-07-30';
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(0);
  });

  it('should not validate a secondDate prior to firstDate', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.firstDate = '2023-07-20';
    myClassInstance.secondDate = '2023-07-19';
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(1);
  });

  it('should not validate if dates are invalid', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.firstDate = '2023-07-40';
    myClassInstance.secondDate = '2023-07-19';
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(1);
  });
});
