import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { AD_ROUTE_PROVIDER } from '@modules/ad/ad.di-tokens';
import { RouteProviderPort } from '@modules/ad/core/application/ports/route-provider.port';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';
import { JourneyItem } from '@modules/ad/core/domain/value-objects/journey-item.value-object';
import { MatchRequestDto } from '@modules/ad/interface/grpc-controllers/dtos/match.request.dto';
import { WaypointDto } from '@modules/ad/interface/grpc-controllers/dtos/waypoint.dto';
import { MatchGrpcController } from '@modules/ad/interface/grpc-controllers/match.grpc-controller';
import { QueryBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const originWaypoint: WaypointDto = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: WaypointDto = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const punctualMatchRequestDto: MatchRequestDto = {
  driver: false,
  passenger: true,
  frequency: Frequency.PUNCTUAL,
  fromDate: '2023-08-15',
  toDate: '2023-08-15',
  schedule: [
    {
      time: '07:00',
      day: 2,
      margin: 900,
    },
  ],
  waypoints: [originWaypoint, destinationWaypoint],
  strict: false,
  algorithmType: AlgorithmType.PASSENGER_ORIENTED,
};

const mockQueryBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => [
      MatchEntity.create({
        adId: '53a0bf71-4132-4f7b-a4cc-88c796b6bdf1',
        role: Role.DRIVER,
        distance: 356041,
        duration: 12647,
        journeys: [
          {
            firstDate: new Date('2023-09-01'),
            lastDate: new Date('2024-08-30'),
            journeyItems: [
              new JourneyItem({
                lat: 48.689445,
                lon: 6.17651,
                duration: 0,
                distance: 0,
                actorTimes: [
                  new ActorTime({
                    role: Role.DRIVER,
                    target: Target.START,
                    firstDatetime: new Date('2023-09-01 07:00'),
                    firstMinDatetime: new Date('2023-09-01 06:45'),
                    firstMaxDatetime: new Date('2023-09-01 07:15'),
                    lastDatetime: new Date('2024-08-30 07:00'),
                    lastMinDatetime: new Date('2024-08-30 06:45'),
                    lastMaxDatetime: new Date('2024-08-30 07:15'),
                  }),
                ],
              }),
              new JourneyItem({
                lat: 48.369445,
                lon: 6.67487,
                duration: 2100,
                distance: 56878,
                actorTimes: [
                  new ActorTime({
                    role: Role.DRIVER,
                    target: Target.NEUTRAL,
                    firstDatetime: new Date('2023-09-01 07:35'),
                    firstMinDatetime: new Date('2023-09-01 07:20'),
                    firstMaxDatetime: new Date('2023-09-01 07:50'),
                    lastDatetime: new Date('2024-08-30 07:35'),
                    lastMinDatetime: new Date('2024-08-30 07:20'),
                    lastMaxDatetime: new Date('2024-08-30 07:50'),
                  }),
                  new ActorTime({
                    role: Role.PASSENGER,
                    target: Target.START,
                    firstDatetime: new Date('2023-09-01 07:32'),
                    firstMinDatetime: new Date('2023-09-01 07:17'),
                    firstMaxDatetime: new Date('2023-09-01 07:47'),
                    lastDatetime: new Date('2024-08-30 07:32'),
                    lastMinDatetime: new Date('2024-08-30 07:17'),
                    lastMaxDatetime: new Date('2024-08-30 07:47'),
                  }),
                ],
              }),
              new JourneyItem({
                lat: 47.98487,
                lon: 6.9427,
                duration: 3840,
                distance: 76491,
                actorTimes: [
                  new ActorTime({
                    role: Role.DRIVER,
                    target: Target.NEUTRAL,
                    firstDatetime: new Date('2023-09-01 08:04'),
                    firstMinDatetime: new Date('2023-09-01 07:51'),
                    firstMaxDatetime: new Date('2023-09-01 08:19'),
                    lastDatetime: new Date('2024-08-30 08:04'),
                    lastMinDatetime: new Date('2024-08-30 07:51'),
                    lastMaxDatetime: new Date('2024-08-30 08:19'),
                  }),
                  new ActorTime({
                    role: Role.PASSENGER,
                    target: Target.FINISH,
                    firstDatetime: new Date('2023-09-01 08:01'),
                    firstMinDatetime: new Date('2023-09-01 07:46'),
                    firstMaxDatetime: new Date('2023-09-01 08:16'),
                    lastDatetime: new Date('2024-08-30 08:01'),
                    lastMinDatetime: new Date('2024-08-30 07:46'),
                    lastMaxDatetime: new Date('2024-08-30 08:16'),
                  }),
                ],
              }),
              new JourneyItem({
                lat: 47.365987,
                lon: 7.02154,
                duration: 4980,
                distance: 96475,
                actorTimes: [
                  new ActorTime({
                    role: Role.DRIVER,
                    target: Target.FINISH,
                    firstDatetime: new Date('2023-09-01 08:23'),
                    firstMinDatetime: new Date('2023-09-01 08:08'),
                    firstMaxDatetime: new Date('2023-09-01 08:38'),
                    lastDatetime: new Date('2024-08-30 08:23'),
                    lastMinDatetime: new Date('2024-08-30 08:08'),
                    lastMaxDatetime: new Date('2024-08-30 08:38'),
                  }),
                ],
              }),
            ],
          },
        ],
      }),
    ])
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockRouteProvider: RouteProviderPort = {
  getBasic: jest.fn(),
  getDetailed: jest.fn(),
};

describe('Match Grpc Controller', () => {
  let matchGrpcController: MatchGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        MatchGrpcController,
        {
          provide: QueryBus,
          useValue: mockQueryBus,
        },
        {
          provide: AD_ROUTE_PROVIDER,
          useValue: mockRouteProvider,
        },
      ],
    }).compile();

    matchGrpcController = module.get<MatchGrpcController>(MatchGrpcController);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(matchGrpcController).toBeDefined();
  });

  it('should return matches', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    const matchPaginatedResponseDto = await matchGrpcController.match(
      punctualMatchRequestDto,
    );
    expect(matchPaginatedResponseDto.data).toHaveLength(1);
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    expect.assertions(3);
    try {
      await matchGrpcController.match(punctualMatchRequestDto);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
  });
});
